package GasTankStress "Model to analyze the stresses that a pressure vessel experiences from expanding and contracting gases."
  package Components "Package containing components used in the example"
    extends Modelica.Icons.Package;

    model FixedVolume "Fixed volume representing a gas tank"
      replaceable package Medium = Modelica.Media.Interfaces.PartialMedium "Medium model" annotation(choicesAllMatching = true);
      parameter Modelica.Units.SI.Volume V = 1e-6 "Fixed size of volume";
      final parameter Boolean use_p_start = true "Select p_start or d_start" annotation(Evaluate = true);
      parameter Medium.AbsolutePressure p_start = 101325 "Initial pressure" annotation(Dialog(group = "Initial Values", enable = use_p_start));
      final parameter Boolean use_T_start = true "Select T_start or h_start" annotation(Evaluate = true);
      parameter Medium.Temperature T_start = 293.15 "Initial temperature" annotation(Dialog(group = "Initial Values", enable = use_T_start));
      Medium.BaseProperties medium(preferredMediumStates = true);
      Modelica.Units.SI.Energy U "Internal energy of port volume";
      Modelica.Units.SI.Mass m "Mass of junction volume";
      Modelica.Units.SI.Mass mXi[Medium.nXi] "Independent substance masses of junction volume";
      Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_a port_a annotation(Placement(visible = true, transformation(origin = {4.99, -96.464}, extent = {{-10, -10}, {10, 10}}, rotation = 0), iconTransformation(origin = {0, 100}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
    initial equation
      medium.p = p_start;
      medium.T = T_start;
      medium.Xi = fill(1 / Medium.nXi, Medium.nXi);
    equation
      // Total quantities
      m = V * medium.d;
      mXi = m * medium.Xi;
      U = m * medium.u;
      // Mass and energy balance
      der(m) = 0;
      der(mXi) = fill(0, Medium.nXi);
      der(U) = port_a.Q_flow;
      port_a.T = medium.T;
      annotation(Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}, initialScale = 0.1, grid = {10, 10}), graphics = {Ellipse(visible = true, origin = {-0, 20}, lineColor = {10, 90, 224}, fillColor = {170, 213, 255}, fillPattern = FillPattern.Sphere, extent = {{-80, -80}, {80, 80}}), Text(visible = true, textColor = {64, 64, 64}, extent = {{-150, 110}, {150, 150}}, textString = "%name"), Text(visible = true, textColor = {64, 64, 64}, extent = {{-150, -150}, {150, -110}}, textString = "V=%V"), Polygon(visible = true, origin = {-0, -75}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid, points = {{-100, -25}, {-40, 25}, {40, 25}, {100, -25}})}), Documentation(info = "<html><div>
<div>This component represents a gas tank or pressure vessel of fixed volume.</div>
</div>
<div>
<div>
<h4>Parameters</h4>
</div>
</div>
<div>
<div>
<p>The gas or liquid medium that is present in the vessel can be changed by altering the Medium parameter. You can access a drop-down list of preset choices by clicking the small down arrow in the input field. The choice of medium will influence the heat capacity of the vessel, as well as the relationship between temperature and pressure.</p>
</div>
</div>
<div>
<div>
<p><img src=\"modelica://GasTankStress/DocumentationFiles/mediumbar.png\" alt=\"mediumbar\" /></p>
</div>
</div>
<div>
<div>
<p>The parameters p_start and T_start will influence the starting conditions for the vessel.</p>
</div>
</div></html>"));
    end FixedVolume;

    model Environment "Heat source containing a table of ambient temperatures"
      Modelica.Thermal.HeatTransfer.Sources.PrescribedTemperature prescribedTemperature annotation(Placement(visible = true, transformation(origin = {50, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
      Modelica.Thermal.HeatTransfer.Interfaces.HeatPort_b port annotation(Placement(visible = true, transformation(origin = {100, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 0), iconTransformation(origin = {0, -100}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
      replaceable model TimeData = Modelica.Blocks.Sources.CombiTimeTable constrainedby Modelica.Blocks.Sources.CombiTimeTable;
      TimeData timeData(table = {{0, 5743 / 20}, {86400 / 47, 5723 / 20}, {172800 / 47, 5723 / 20}, {259200 / 47, 5703 / 20}, {345600 / 47, 5703 / 20}, {432000 / 47, 5703 / 20}, {518400 / 47, 5703 / 20}, {604800 / 47, 5703 / 20}, {691200 / 47, 5723 / 20}, {777600 / 47, 5723 / 20}, {864000 / 47, 5723 / 20}, {950400 / 47, 5743 / 20}, {1036800 / 47, 5743 / 20}, {1123200 / 47, 5763 / 20}, {1209600 / 47, 5783 / 20}, {1296000 / 47, 5803 / 20}, {1382400 / 47, 5823 / 20}, {1468800 / 47, 5843 / 20}, {1555200 / 47, 5863 / 20}, {1641600 / 47, 5883 / 20}, {1728000 / 47, 5903 / 20}, {1814400 / 47, 5903 / 20}, {1900800 / 47, 5923 / 20}, {1987200 / 47, 5923 / 20}, {2073600 / 47, 5943 / 20}, {2160000 / 47, 5943 / 20}, {2246400 / 47, 5943 / 20}, {2332800 / 47, 5943 / 20}, {2419200 / 47, 5943 / 20}, {2505600 / 47, 5943 / 20}, {2592000 / 47, 5943 / 20}, {2678400 / 47, 5923 / 20}, {2764800 / 47, 5943 / 20}, {2851200 / 47, 5943 / 20}, {2937600 / 47, 5943 / 20}, {3024000 / 47, 5923 / 20}, {3110400 / 47, 5923 / 20}, {3196800 / 47, 5903 / 20}, {3283200 / 47, 5903 / 20}, {3369600 / 47, 5883 / 20}, {3456000 / 47, 5863 / 20}, {3542400 / 47, 5843 / 20}, {3628800 / 47, 5823 / 20}, {3715200 / 47, 5823 / 20}, {3801600 / 47, 5803 / 20}, {3888000 / 47, 5803 / 20}, {3974400 / 47, 5783 / 20}, {86400, 5763 / 20}}) annotation(Placement(visible = true, transformation(origin = {-70, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
      Modelica.Blocks.Continuous.Filter filter(f_cut = 1 / (60 * 60 * 2)) annotation(Placement(visible = true, transformation(origin = {-10, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
    equation
      connect(prescribedTemperature.port, port) annotation(Line(visible = true, origin = {80, 0}, points = {{-20, -0}, {20, 0}}, color = {191, 0, 0}));
      connect(filter.u, timeData.y[1]) annotation(Line(visible = true, origin = {-40.5, 0}, points = {{18.5, 0}, {-18.5, 0}}, color = {1, 37, 163}));
      connect(prescribedTemperature.T, filter.y) annotation(Line(visible = true, origin = {19.5, 0}, points = {{18.5, 0}, {-18.5, 0}}, color = {1, 37, 163}));
      annotation(Documentation(info = "<html><div>
<div>Heat source containing a table of ambient temperatures. The data is from a spring day in Link&ouml;ping, Sweden and has been gathered using Wolfram|Alpha. The data is filtered and converted into kelvin.</div>
</div></html>"), Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10}), graphics = {Bitmap(visible = true, fileName = "modelica://GasTankStress.Components.Environment/DocumentationFiles/15148-illustration-of-the-sun-with-clouds-pv.png", imageSource = "", extent = {{-100, -100}, {100, 100}}), Text(visible = true, textColor = {64, 64, 64}, extent = {{-150, 110}, {150, 150}}, textString = "%name")}), Diagram(coordinateSystem(extent = {{-101.665, -35}, {120, 35}}, preserveAspectRatio = true, initialScale = 0.1, grid = {5, 5})));
    end Environment;
    annotation(Documentation(info = "<html></html>"), Diagram(coordinateSystem(extent = {{-148.5, -105.0}, {148.5, 105.0}}, preserveAspectRatio = true, initialScale = 0.1, grid = {5, 5})));
  end Components;

  model GasTankStressModel "Model of the stresses affecting a gas tank"
    extends Modelica.Icons.Example;
    replaceable package Medium = Modelica.Media.IdealGases.SingleGases.H2 constrainedby Modelica.Media.Interfaces.PartialMedium "Medium model" annotation(choicesAllMatching = true);
    Components.FixedVolume pressureVessel(V.displayUnit = "m3", V = vesselVolume, redeclare replaceable package Medium = Medium, p_start.displayUnit = "psi", p_start = 4998699.03754706, T_start = 287.15) annotation(Placement(visible = true, transformation(origin = {0, -32.154}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
    Modelica.Thermal.HeatTransfer.Components.ThermalConductor vesselConduction(G = vesselConductance * vesselSurfaceArea) annotation(Placement(visible = true, transformation(origin = {0, 10}, extent = {{-10, -10}, {10, 10}}, rotation = -1170)));
    parameter Modelica.Units.SI.Volume vesselVolume = 1 "Inner volume of pressure vessel";
    parameter Modelica.Units.SI.Area vesselSurfaceArea = 4 * Modelica.Constants.pi * vesselRadius ^ 2 "Surface area of the vessel" annotation(Dialog(tab = "Other Parameters"));
    parameter Modelica.Units.SI.Length vesselRadius = (vesselVolume * (3 / 4) * (1 / Modelica.Constants.pi)) ^ (1 / 3) "Radius of the vessel" annotation(Dialog(tab = "Other Parameters"));
    parameter Real vesselConductance(unit = "(W/K)/m2") = 80 "Thermal conductance of the vessel shell" annotation(choices(choice = 16 "Material: Stainless Steel", choice = 54 "Material: Low Carbon Steel", choice = 80 "Material: Iron", choice = 250 "Material: Aluminium"));
    parameter Modelica.Units.SI.Pressure pressureLimitWarning(displayUnit = "psi") = 5171067.96987627 "Pressure limit where a warning is issued" annotation(Dialog(group = "Pressure Limits"));
    parameter Modelica.Units.SI.Pressure pressureLimitCritical(displayUnit = "psi") = 5515805.83453469 "Pressure limit where risk of structural failure critical" annotation(Dialog(group = "Pressure Limits"));
    Components.Environment environment(timeData.smoothness = Modelica.Blocks.Types.Smoothness.ContinuousDerivative) annotation(Placement(visible = true, transformation(origin = {0, 50}, extent = {{-10, -10}, {10, 10}}, rotation = -360)));
  equation
    connect(vesselConduction.port_b, pressureVessel.port_a) annotation(Line(visible = true, origin = {0, -11.077}, points = {{0, 11.077}, {-0, -11.077}}, color = {191, 0, 0}));
    connect(environment.port, vesselConduction.port_a) annotation(Line(visible = true, origin = {0, 30}, points = {{-0, 10}, {0, -10}}, color = {191, 0, 0}));
    annotation(preferredView = "info", __Wolfram(ControlPanels(Panel(identifier = "recursing-neumann", title = "Model parameters", elements = {Slider(variable = pressureLimitCritical, label = "Pressure limit where risk of structural failure critical", min = 4136854.37590102, max = 6894757.29316836), Slider(variable = pressureLimitWarning, label = "Pressure limit where a warning is issued", min = 4136854.37590102, max = 6894757.29316836), Slider(variable = vesselConductance, label = "Thermal conductance of the vessel shell", min = 50, max = 100), Slider(variable = vesselVolume, label = "Inner volume of pressure vessel", min = 0.1, max = 20)}))), experiment(StopTime = 86400), Documentation(info = "<html>
<p>This model analyzes the stresses that a pressure vessel experiences from expanding and contracting gases. If a pressure vessel is filled with a gas that is colder than the ambient temperature, the pressure inside the vessel will increase as heat is absorbed. This can lead to significant variations in pressure levels, especially if it is exposed to a varying ambient temperature.</p>

<h4><span style=\"font-size: 16px;\">Changing Parameters</span></h4>

<p>The gas or liquid medium that is present in the vessel can be changed by altering the Medium parameter. You can access a drop-down list of preset choices by clicking the small down arrow in the input field. The choice of medium will influence the heat capacity of the vessel, as well as the relationship between temperature and pressure.</p>

<p style=\"text-align: center;\"><img src=\"modelica://GasTankStress/DocumentationFiles/selectingmedia.png\" style=\"width:656px\" alt=\"selectingmedia\" />
<h4>Ambient Temperature</h4>

<p>In the model, the ambient temperature is determined by a data table with preset values. The data is from a spring day in Link&ouml;ping, Sweden and has been gathered using Wolfram|Alpha.</p>

<p style=\"text-align: center;\"><img src=\"modelica://GasTankStress/DocumentationFiles/ambienttemperature.png\" style=\"width:565px\" alt=\"ambienttemperature\" /></p>

<p>Heat transfer between the ambient environment and the content of the pressure vessel is determined by the conductance of the vessel material. Different materials can be selected by accessing the drop-down menu in the VesselConductance parameter.</p>

<p style=\"text-align: center;\"><img src=\"modelica://GasTankStress/DocumentationFiles/materialconductance.png\" style=\"width:555px\" alt=\"materialconductance\" /></p>

<h4>Plot the Results</h4>

<p>Click the Simulate button: <img src=\"modelica://GasTankStress/DocumentationFiles/play.svg\" style=\"width:20px\" alt=\"simulate\" /></p>

<p>After simulating the model, you will see the Gas Pressure and Limits plot:</p>


<p style=\"text-align: center;\"><img src=\"modelica://GasTankStress/DocumentationFiles/gasandpressurelimits.png\" style=\"width:600px\" alt=\"\" /></p>

<p>This graph explores how the pressure varies during the day and how it relates to the pressure limits set in the model.</p>

<p>Other stored plots can be accessed in the Stored Plots section of the Experiment Browser. The Temperatures plot shows the gas and ambient temperature variations during the day. The higher the heat conductivity of the pressure vessel, the more closely the two temperatures will follow each other. A gas with lower heat capacity will also adjust more quickly to the ambient temperature.</p>

<p>The Pressure vs. Temperature plot displays a parametric plot of gas temperatures versus gas pressures. If an ideal gas is used, this will be a straight line.</p>


<p style=\"text-align: center;\"><img src=\"modelica://GasTankStress/DocumentationFiles/pressurevstemperature.png\" style=\"width:480px\" alt=\"\" /></p>
</html>", figures = {Figure(title = "Gas pressure and limits", identifier = "f7e43", preferred = true, plots = {Plot(curves = {Curve(y = pressureVessel.medium.p, legend = "Gas Pressure"), Curve(y = pressureLimitWarning, legend = "Warning Pressure"), Curve(y = pressureLimitCritical, legend = "Critical Pressure")}, x = Axis(unit = "h"), y = Axis(unit = "psi"))}), Figure(title = "Temperatures", identifier = "43f25", plots = {Plot(curves = {Curve(y = pressureVessel.medium.T, legend = "Gas Temperature"), Curve(y = environment.port.T, legend = "Environment Temperature")}, x = Axis(unit = "h"))}), Figure(title = "Pressure vs temperature", identifier = "6a08f", plots = {Plot(curves = {Curve(x = pressureVessel.medium.T, y = pressureVessel.medium.p, legend = "Pressure vs Temperature")}, y = Axis(unit = "psi"))})}), Diagram(coordinateSystem(extent = {{-40, -60}, {40, 70}}, preserveAspectRatio = true, initialScale = 0.1, grid = {5, 5})));
  end GasTankStressModel;
  annotation(preferredView = "info", uses(Modelica(version = "4.0.0")), __Wolfram(openClass = GasTankStress.GasTankStressModel), Documentation(info = "<html></html>"), Icon(coordinateSystem(extent = {{-100.0, -100.0}, {100.0, 100.0}}, preserveAspectRatio = true, initialScale = 0.1, grid = {10, 10}), graphics = {Rectangle(visible = true, lineColor = {111, 0, 0}, fillColor = {255, 0, 0}, fillPattern = FillPattern.HorizontalCylinder, lineThickness = 4, extent = {{-90.0, -90.0}, {90.0, 90.0}}, radius = 25), Text(visible = true, origin = {12.0, 20.8333}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, extent = {{-102.0, -70.8333}, {78.0, 29.1667}}, textString = "Ex", fontName = "Arial")}));
end GasTankStress;
